@@ -3,11 +3,11 @@ use chrono::prelude::*;
 use encoding_rs::Encoding;
 use html5ever::interface::QualName;
 use html5ever::parse_document;
-use html5ever::rcdom::{Handle, NodeData, RcDom};
 use html5ever::serialize::{serialize, SerializeOpts};
 use html5ever::tendril::{format_tendril, TendrilSink};
 use html5ever::tree_builder::{Attribute, TreeSink};
 use html5ever::{local_name, namespace_url, ns, LocalName};
+use markup5ever_rcdom::{Handle, NodeData, RcDom, SerializableHandle};
 use regex::Regex;
 use reqwest::blocking::Client;
 use reqwest::Url;
@@ -30,10 +30,16 @@ struct SrcSetItem<'a> {
 
 const ICON_VALUES: &'static [&str] = &["icon", "shortcut icon"];
 
+const WHITESPACES: &'static [char] = &['\t', '\n', '\x0c', '\r', ' '];
+
 pub fn add_favicon(document: &Handle, favicon_data_url: String) -> RcDom {
     let mut buf: Vec<u8> = Vec::new();
-    serialize(&mut buf, document, SerializeOpts::default())
-        .expect("unable to serialize DOM into buffer");
+    serialize(
+        &mut buf,
+        &SerializableHandle::from(document.clone()),
+        SerializeOpts::default(),
+    )
+    .expect("unable to serialize DOM into buffer");
 
     let mut dom = html_to_dom(&buf, "utf-8".to_string());
     let doc = dom.get_document();
@@ -161,15 +167,44 @@ pub fn embed_srcset(
     document_url: &Url,
     srcset: &str,
     options: &Options,
-    depth: u32,
 ) -> String {
     let mut array: Vec<SrcSetItem> = vec![];
-    let re = Regex::new(r",\s+").unwrap();
-    for srcset_item in re.split(srcset) {
-        let parts: Vec<&str> = srcset_item.trim().split_whitespace().collect();
-        if parts.len() > 0 {
-            let path = parts[0].trim();
-            let descriptor = if parts.len() > 1 { parts[1].trim() } else { "" };
+
+    // Parse srcset attribute according to the specs
+    // https://html.spec.whatwg.org/multipage/images.html#srcset-attribute
+    let mut offset = 0;
+    let size = srcset.chars().count();
+
+    while offset < size {
+        let mut has_descriptor = true;
+        // Zero or more whitespaces + skip leading comma
+        let url_start = offset
+            + srcset[offset..]
+                .chars()
+                .take_while(|&c| WHITESPACES.contains(&c) || c == ',')
+                .count();
+        if url_start >= size {
+            break;
+        }
+        // A valid non-empty URL that does not start or end with comma
+        let mut url_end = url_start
+            + srcset[url_start..]
+                .chars()
+                .take_while(|&c| !WHITESPACES.contains(&c))
+                .count();
+        while (url_end - 1) > url_start && srcset.chars().nth(url_end - 1).unwrap() == ',' {
+            has_descriptor = false;
+            url_end -= 1;
+        }
+        offset = url_end;
+        // If the URL wasn't terminated by comma there may also be a descriptor
+        if has_descriptor {
+            offset += srcset[url_end..].chars().take_while(|&c| c != ',').count();
+        }
+        // Collect SrcSetItem
+        if url_end > url_start {
+            let path = &srcset[url_start..url_end];
+            let descriptor = &srcset[url_end..offset].trim();
             let srcset_real_item = SrcSetItem { path, descriptor };
             array.push(srcset_real_item);
         }
@@ -182,14 +217,7 @@ pub fn embed_srcset(
             result.push_str(EMPTY_IMAGE_DATA_URL);
         } else {
             let image_full_url: Url = resolve_url(&document_url, part.path);
-            match retrieve_asset(
-                cache,
-                client,
-                &document_url,
-                &image_full_url,
-                options,
-                depth + 1,
-            ) {
+            match retrieve_asset(cache, client, &document_url, &image_full_url, options) {
                 Ok((image_data, image_final_url, image_media_type, image_charset)) => {
                     let mut image_data_url = create_data_url(
                         &image_media_type,
@@ -197,7 +225,7 @@ pub fn embed_srcset(
                         &image_data,
                         &image_final_url,
                     );
-                    // Append retreved asset as a data URL
+                    // Append retrieved asset as a data URL
                     image_data_url.set_fragment(image_full_url.fragment());
                     result.push_str(image_data_url.as_ref());
                 }
@@ -428,8 +456,12 @@ pub fn is_icon(attr_value: &str) -> bool {
 
 pub fn set_base_url(document: &Handle, desired_base_href: String) -> RcDom {
     let mut buf: Vec<u8> = Vec::new();
-    serialize(&mut buf, document, SerializeOpts::default())
-        .expect("unable to serialize DOM into buffer");
+    serialize(
+        &mut buf,
+        &SerializableHandle::from(document.clone()),
+        SerializeOpts::default(),
+    )
+    .expect("unable to serialize DOM into buffer");
 
     let mut dom = html_to_dom(&buf, "utf-8".to_string());
     let doc = dom.get_document();
@@ -534,7 +566,7 @@ pub fn set_node_attr(node: &Handle, attr_name: &str, attr_value: Option<String>)
 
 pub fn serialize_document(mut dom: RcDom, document_encoding: String, options: &Options) -> Vec<u8> {
     let mut buf: Vec<u8> = Vec::new();
-    let doc = dom.get_document();
+    let document = dom.get_document();
 
     if options.isolate
         || options.no_css
@@ -544,7 +576,7 @@ pub fn serialize_document(mut dom: RcDom, document_encoding: String, options: &O
         || options.no_images
     {
         // Take care of CSP
-        if let Some(html) = get_child_node_by_name(&doc, "html") {
+        if let Some(html) = get_child_node_by_name(&document, "html") {
             if let Some(head) = get_child_node_by_name(&html, "head") {
                 let meta = dom.create_element(
                     QualName::new(None, ns!(), local_name!("meta")),
@@ -570,8 +602,12 @@ pub fn serialize_document(mut dom: RcDom, document_encoding: String, options: &O
         }
     }
 
-    serialize(&mut buf, &doc, SerializeOpts::default())
-        .expect("Unable to serialize DOM into buffer");
+    serialize(
+        &mut buf,
+        &SerializableHandle::from(document.clone()),
+        SerializeOpts::default(),
+    )
+    .expect("Unable to serialize DOM into buffer");
 
     // Unwrap NOSCRIPT elements
     if options.unwrap_noscript {
@@ -599,18 +635,10 @@ pub fn retrieve_and_embed_asset(
     attr_name: &str,
     attr_value: &str,
     options: &Options,
-    depth: u32,
 ) {
     let resolved_url: Url = resolve_url(document_url, attr_value);
 
-    match retrieve_asset(
-        cache,
-        client,
-        &document_url.clone(),
-        &resolved_url,
-        options,
-        depth + 1,
-    ) {
+    match retrieve_asset(cache, client, &document_url.clone(), &resolved_url, options) {
         Ok((data, final_url, mut media_type, charset)) => {
             let node_name: &str = get_node_name(&node).unwrap();
 
@@ -639,7 +667,7 @@ pub fn retrieve_and_embed_asset(
 
                 if node_name == "link" && determine_link_node_type(node) == "stylesheet" {
                     // Stylesheet LINK elements require special treatment
-                    let css: String = embed_css(cache, client, &final_url, &s, options, depth + 1);
+                    let css: String = embed_css(cache, client, &final_url, &s, options);
 
                     // Create and embed data URL
                     let css_data_url =
@@ -648,19 +676,12 @@ pub fn retrieve_and_embed_asset(
                 } else if node_name == "frame" || node_name == "iframe" {
                     // (I)FRAMEs are also quite different from conventional resources
                     let frame_dom = html_to_dom(&data, charset.clone());
-                    walk_and_embed_assets(
-                        cache,
-                        client,
-                        &final_url,
-                        &frame_dom.document,
-                        &options,
-                        depth + 1,
-                    );
+                    walk_and_embed_assets(cache, client, &final_url, &frame_dom.document, &options);
 
                     let mut frame_data: Vec<u8> = Vec::new();
                     serialize(
                         &mut frame_data,
-                        &frame_dom.document,
+                        &SerializableHandle::from(frame_dom.document.clone()),
                         SerializeOpts::default(),
                     )
                     .unwrap();
@@ -710,13 +731,12 @@ pub fn walk_and_embed_assets(
     document_url: &Url,
     node: &Handle,
     options: &Options,
-    depth: u32,
 ) {
     match node.data {
         NodeData::Document => {
             // Dig deeper
             for child in node.children.borrow().iter() {
-                walk_and_embed_assets(cache, client, &document_url, child, options, depth);
+                walk_and_embed_assets(cache, client, &document_url, child, options);
             }
         }
         NodeData::Element {
@@ -751,7 +771,6 @@ pub fn walk_and_embed_assets(
                                     "href",
                                     &link_attr_href_value,
                                     options,
-                                    depth,
                                 );
                             } else {
                                 set_node_attr(node, "href", None);
@@ -774,7 +793,6 @@ pub fn walk_and_embed_assets(
                                         "href",
                                         &link_attr_href_value,
                                         options,
-                                        depth,
                                     );
                                 }
                             }
@@ -816,7 +834,6 @@ pub fn walk_and_embed_assets(
                                 "background",
                                 &body_attr_background_value,
                                 options,
-                                depth,
                             );
                         }
                     }
@@ -862,7 +879,6 @@ pub fn walk_and_embed_assets(
                                 "src",
                                 &img_full_url,
                                 options,
-                                depth,
                             );
                         }
                     }
@@ -870,14 +886,8 @@ pub fn walk_and_embed_assets(
                     // Resolve srcset attribute
                     if let Some(img_srcset) = get_node_attr(node, "srcset") {
                         if !img_srcset.is_empty() {
-                            let resolved_srcset: String = embed_srcset(
-                                cache,
-                                client,
-                                &document_url,
-                                &img_srcset,
-                                options,
-                                depth,
-                            );
+                            let resolved_srcset: String =
+                                embed_srcset(cache, client, &document_url, &img_srcset, options);
                             set_node_attr(node, "srcset", Some(resolved_srcset));
                         }
                     }
@@ -907,7 +917,6 @@ pub fn walk_and_embed_assets(
                                         "src",
                                         &input_attr_src_value,
                                         options,
-                                        depth,
                                     );
                                 }
                             }
@@ -940,7 +949,6 @@ pub fn walk_and_embed_assets(
                             "href",
                             &image_href,
                             options,
-                            depth,
                         );
                     }
                 }
@@ -961,7 +969,6 @@ pub fn walk_and_embed_assets(
                                     "src",
                                     &source_attr_src_value,
                                     options,
-                                    depth,
                                 );
                             }
                         } else if parent_node_name == "video" {
@@ -976,7 +983,6 @@ pub fn walk_and_embed_assets(
                                     "src",
                                     &source_attr_src_value,
                                     options,
-                                    depth,
                                 );
                             }
                         }
@@ -998,7 +1004,6 @@ pub fn walk_and_embed_assets(
                                         &document_url,
                                         &source_attr_srcset_value,
                                         options,
-                                        depth,
                                     );
                                     set_node_attr(node, "srcset", Some(resolved_srcset));
                                 }
@@ -1051,7 +1056,6 @@ pub fn walk_and_embed_assets(
                             "src",
                             &script_attr_src.unwrap_or_default(),
                             options,
-                            depth,
                         );
                     }
                 }
@@ -1069,7 +1073,6 @@ pub fn walk_and_embed_assets(
                                     &document_url,
                                     tendril.as_ref(),
                                     options,
-                                    depth,
                                 );
                                 tendril.clear();
                                 tendril.push_slice(&replacement);
@@ -1101,7 +1104,6 @@ pub fn walk_and_embed_assets(
                                     "src",
                                     &frame_attr_src_value,
                                     options,
-                                    depth,
                                 );
                             }
                         }
@@ -1121,7 +1123,6 @@ pub fn walk_and_embed_assets(
                                 "src",
                                 &audio_attr_src_value,
                                 options,
-                                depth,
                             );
                         }
                     }
@@ -1140,7 +1141,6 @@ pub fn walk_and_embed_assets(
                                 "src",
                                 &video_attr_src_value,
                                 options,
-                                depth,
                             );
                         }
                     }
@@ -1164,7 +1164,6 @@ pub fn walk_and_embed_assets(
                                     "poster",
                                     &video_attr_poster_value,
                                     options,
-                                    depth,
                                 );
                             }
                         }
@@ -1188,7 +1187,6 @@ pub fn walk_and_embed_assets(
                                     &document_url,
                                     &noscript_contents_dom.document,
                                     &options,
-                                    depth,
                                 );
                                 // Get rid of original contents
                                 noscript_contents.clear();
@@ -1198,8 +1196,12 @@ pub fn walk_and_embed_assets(
                                 {
                                     if let Some(body) = get_child_node_by_name(&html, "body") {
                                         let mut buf: Vec<u8> = Vec::new();
-                                        serialize(&mut buf, &body, SerializeOpts::default())
-                                            .expect("Unable to serialize DOM into buffer");
+                                        serialize(
+                                            &mut buf,
+                                            &SerializableHandle::from(body.clone()),
+                                            SerializeOpts::default(),
+                                        )
+                                        .expect("Unable to serialize DOM into buffer");
                                         let result = String::from_utf8_lossy(&buf);
                                         noscript_contents.push_slice(&result);
                                     }
@@ -1225,7 +1227,6 @@ pub fn walk_and_embed_assets(
                         &document_url,
                         &node_attr_style_value,
                         options,
-                        depth,
                     );
                     set_node_attr(node, "style", Some(embedded_style));
                 }
@@ -1249,7 +1250,7 @@ pub fn walk_and_embed_assets(
 
             // Dig deeper
             for child in node.children.borrow().iter() {
-                walk_and_embed_assets(cache, client, &document_url, child, options, depth);
+                walk_and_embed_assets(cache, client, &document_url, child, options);
             }
         }
         _ => {
